/* 
===============================================================================
=== Released under GPL (GNU Public License) ==== No Any Warranty ==============
=============================================================================== 
*/ 
 
#include "../include/common.h"  
#include "../include/filebrowser.h" 

#define MENU_POPUP_ITEMS 3

#define OZEDIT_DIRTREE_DIR 1 
#define OZEDIT_DIRTREE_FILE 2
#define OZEDIT_DIRTREE_LINK 3
#define OZEDIT_DIRTREE_DEVICE 4


static char *file_browser_popup_menu_items[MENU_POPUP_ITEMS][2] = {
	"Go Here","go_here",
	"Rename","rename_object",
	"Clear","clear_object"
};


GtkTreeIter filebrowser_selected_iter; /* Contiene l'ITER dell'Oggetto Selezionato */
GtkTreeStore *filetreestore;


struct _OzEDIT_DirObj {
    gchar name[255];    /* Nome completo (+ path) - Della Directory o del File */
    gint type;          /* Tipo dell'oggetto */
    gboolean loaded;	/* Directory caricata ? */
    GtkTreeIter iter;   /* ITER nella lista */
    GList *subdir;      /* Se directory questa lista contiene gli oggetti sotto */
} *dirtree;

void OzEDIT_FileBrowserOpenDir(struct _OzEDIT_DirObj *,GtkTreeIter *);

/*

/ -+- bin
   |
   +- usr -+- local -+- share
   |       |         |
           |         +- bin
           +- bin

etc etc etc

*/

/**************** CALLBACKS ***********************************************************************************************/

static void OzEDIT_CBFileBrowserPopupMenuActivate(gchar *menu) {        
#ifdef DEBUG
    g_print("FileBrowser POPUP Menu %s \n",menu);
#endif    
}

GtkWidget *OzEDIT_FileBrowserCreatePopupMenu() {
	GtkWidget *menu,*menu_item;
	gint i;
	
	menu = gtk_menu_new(); 
	
	for(i=0;i<MENU_POPUP_ITEMS;i++) {
		menu_item = gtk_menu_item_new_with_label((gchar *)file_browser_popup_menu_items[i][0]);
		gtk_menu_shell_append(GTK_MENU_SHELL(menu), menu_item);
		g_signal_connect_swapped(G_OBJECT(menu_item), "activate", G_CALLBACK(OzEDIT_CBFileBrowserPopupMenuActivate),(gpointer)file_browser_popup_menu_items[i][1]);
		gtk_widget_show(menu_item);
	}
	return menu;
}

static gint OzEDIT_CBFileBrowserPopupMenu(GtkWidget *widget, GdkEvent *event,GtkWidget *filetree)
{
	GtkWidget *menu;
	GdkEventButton *event_button;
	GtkTreeSelection *tree_selection;
	GtkTreeModel *tree_model;
	gint *indices,c;
	struct _OzEDIT_DirObj *node,*dtree;
	
    
	g_return_val_if_fail(widget != NULL, FALSE);
	g_return_val_if_fail(event != NULL, FALSE);
	
	tree_model = gtk_tree_view_get_model(GTK_TREE_VIEW(filetree));
	tree_selection = gtk_tree_view_get_selection(GTK_TREE_VIEW(filetree));
	if(!gtk_tree_selection_get_selected(tree_selection,NULL,&filebrowser_selected_iter)) {
	    return;
	}

	
	indices = gtk_tree_path_get_indices(gtk_tree_model_get_path(tree_model,&filebrowser_selected_iter));
	
	node = dirtree;
	
	for(c=0;c<gtk_tree_path_get_depth(gtk_tree_model_get_path(tree_model,&filebrowser_selected_iter));c++) {
	    node = g_list_nth_data(node->subdir,indices[c]);
	}
	
#ifdef DEBUG
	g_print("DEBUG: Selected path %s [%s]\n",gtk_tree_path_to_string(gtk_tree_model_get_path(tree_model,&filebrowser_selected_iter)),node->name);
#endif
	
	/* Se click con DESTRO ... */
	if(event->type == GDK_BUTTON_PRESS) {
		event_button = (GdkEventButton *)event;
		if (event_button->button == 3)
		{
			menu = OzEDIT_FileBrowserCreatePopupMenu();
	  		gtk_menu_popup(GTK_MENU(menu),NULL,NULL,NULL,NULL,event_button->button,event_button->time);
	 		return TRUE;
		}
	}
	/* Doppio click con SINISTRO ! */
	if(event->type == GDK_2BUTTON_PRESS)	{
		event_button = (GdkEventButton *)event;
		if (event_button->button == 1)
		{
	    	    /* Se directory ESPANDI, altrimenti APRI file */
		    switch(node->type) {
			case OZEDIT_DIRTREE_DIR: 
				if(node->loaded == FALSE) OzEDIT_FileBrowserOpenDir(node,&node->iter);
				gtk_tree_view_expand_to_path(GTK_TREE_VIEW(filetree),gtk_tree_model_get_path(tree_model,&filebrowser_selected_iter));
			    break;
			case OZEDIT_DIRTREE_FILE:
				OzEDIT_RecentAddFile((gchar *)node->name);
				OzEDIT_CreateNewObject((gchar *)node->name);
			    break;
			default:
			    break;		
		    }
    		}
	}
	return FALSE;
}

void OzEDIT_CBFileBrowserRowActivated(GtkTreeView *tree_view,GtkTreePath *path,GtkTreeViewColumn *column) {
}

/************************************************************************************************************************/
void OzEDIT_FileBrowserOpenDir(struct _OzEDIT_DirObj *thisnode,GtkTreeIter *parentiter) {
    GDir *dir;
    struct _OzEDIT_DirObj *tempnode;
    GtkTreeIter nodeiter;
    gchar *dirname,*path;
    GdkPixbuf *pixbuf_dir,*pixbuf_file;

#ifndef _NOGDKPIXBUF
    pixbuf_dir = gdk_pixbuf_new_from_xpm_data((gchar **)folder_xpm);
    pixbuf_file = gdk_pixbuf_new_from_xpm_data((gchar **)file_xpm);
#endif
        
    dir = g_dir_open(thisnode->name,0,NULL);
    thisnode->loaded = TRUE;
    
    while(1) {
        dirname = g_dir_read_name(dir);
        if(dirname != NULL) {
	    tempnode = g_malloc0(sizeof(struct _OzEDIT_DirObj));
	    g_snprintf(tempnode->name,255,"%s/%s",thisnode->name,dirname);
#ifdef DEBUG
	    g_print("DEBUG: %s\n",tempnode->name);
#endif
	    if(g_file_test(tempnode->name,G_FILE_TEST_IS_DIR)) {
		/* DIRECTORY */
		tempnode->type = OZEDIT_DIRTREE_DIR;
    		gtk_tree_store_append(GTK_TREE_STORE(filetreestore),&tempnode->iter,parentiter);
		gtk_tree_store_set(GTK_TREE_STORE(filetreestore),&tempnode->iter,0,pixbuf_dir,1,dirname,-1);
	    } else {
		/* FILE */
		tempnode->type = OZEDIT_DIRTREE_FILE;	    
    		gtk_tree_store_append(GTK_TREE_STORE(filetreestore),&tempnode->iter,parentiter);
		gtk_tree_store_set(GTK_TREE_STORE(filetreestore),&tempnode->iter,0,pixbuf_file,1,dirname,-1);	    
	    }
	    tempnode->loaded = FALSE;
	    thisnode->subdir = g_list_append(thisnode->subdir,tempnode);
	} else {
	    break;
	}
    }
    g_dir_close(dir);
}

/************ OzEDIT_FileBrowser() - The MAIN Procedure ******************************************************************/
GtkWidget *OzEDIT_FileBrowser() {
	GtkWidget *v_scrollable,*filetree;	
	GtkTreeViewColumn *column;
	GtkCellRenderer *renderer;

	/* Scrollable per la LISTA TREE */
	v_scrollable =  gtk_scrolled_window_new(NULL,NULL);
	gtk_scrolled_window_set_policy(GTK_SCROLLED_WINDOW(v_scrollable),GTK_POLICY_AUTOMATIC,GTK_POLICY_AUTOMATIC);
	gtk_widget_set_size_request(GTK_WIDGET(v_scrollable), -1, 380);
	gtk_container_set_border_width(GTK_CONTAINER(v_scrollable),1);
	gtk_widget_show(v_scrollable);

	/* Tree List Widget */
	filetreestore = gtk_tree_store_new(2,GDK_TYPE_PIXBUF,G_TYPE_STRING);
	filetree = gtk_tree_view_new_with_model(GTK_TREE_MODEL(filetreestore));
	g_object_unref(G_OBJECT(filetreestore));
	gtk_tree_view_set_rules_hint(GTK_TREE_VIEW(filetree), TRUE);
	gtk_tree_selection_set_mode(gtk_tree_view_get_selection(GTK_TREE_VIEW(filetree)),GTK_SELECTION_SINGLE);
	
	column = gtk_tree_view_column_new();
	gtk_tree_view_column_set_title(column,"File Browser");
	
	renderer = gtk_cell_renderer_pixbuf_new();
	gtk_tree_view_column_pack_start(column,renderer,FALSE);
	gtk_tree_view_column_add_attribute(column,renderer,"pixbuf",0);

	renderer = gtk_cell_renderer_text_new();
	gtk_tree_view_column_pack_start(column,renderer,FALSE);
	gtk_tree_view_append_column(GTK_TREE_VIEW(filetree),GTK_TREE_VIEW_COLUMN(column));
	gtk_tree_view_column_add_attribute(column,renderer,"text",1);
	
        g_signal_connect(G_OBJECT(filetree),"button_press_event",G_CALLBACK(OzEDIT_CBFileBrowserPopupMenu),filetree); 
	
	gtk_widget_set_size_request(filetree,200,0);
	gtk_container_add(GTK_CONTAINER(v_scrollable),filetree);

	gtk_widget_show(filetree);

	dirtree = g_malloc0(sizeof(struct _OzEDIT_DirObj));
	g_stpcpy(dirtree->name,"/");
	dirtree->type = OZEDIT_DIRTREE_DIR;
    
	OzEDIT_FileBrowserOpenDir(dirtree,NULL);

	return (GtkWidget *)v_scrollable;
}

