/*
===============================================================================
=== Released under GPL (GNU Public License) ==== No Any Warranty ==============
===============================================================================

Based on the example in the GNet Packet "hfetch.c" :-)

*/

#include "../include/common.h"
#include "../include/ozupdater.h" 

void OzUPDATER_Check_Net();

void OzUPDATER_Check() {
#ifdef G_THREADS_ENABLED
    GThread *thr_updater;
    GError **thr_error;
    if(g_thread_supported()) {
#ifdef DEBUG
	g_print("Check for latest version\n");    
#endif    
	thr_updater = g_thread_create((GThreadFunc)OzUPDATER_Check_Net,NULL,FALSE,thr_error);
    } else {
#ifdef DEBUG
	g_print("Check for latest version (NO THREADED)\n");    
#endif    
	OzUPDATER_Check_Net();
    }
#else
#warning "GThreads NOT Enabled."
    OzUPDATER_Check_Net();
#endif
}


void OzUPDATER_Check_Net() {
#ifndef _NOGNET
    struct _OzGtkXML *XMLRoot, *XMLTemp;
    GInetAddr *addr;
    GTcpSocket *socket;
    GIOChannel *iochannel;
    gchar *command,**rversion,**rdate;
    gchar buffer[2048],delim[5];
    GIOError error;
    gsize bread,bwrite;
    GError *utf8error = NULL;
    guint n;

    g_sprintf(delim,".\0");

    gnet_init ();

    /* Create the address */
    addr = gnet_inetaddr_new(OZUPDATER_URL, 80);
    if(addr == NULL) return;

    /* Create the socket */
    socket = gnet_tcp_socket_new(addr);
    gnet_inetaddr_delete (addr);
    if(socket == NULL) return;

    /* Get the IOChannel */
    iochannel = gnet_tcp_socket_get_io_channel(socket);
    if(iochannel == NULL) return;

    /* Send the command */
    command = g_strconcat("GET ", OZUPDATER_FILE, " HTTP/1.0\n\n", NULL);
#ifdef DEBUG
    g_print("DEBUG: %s -> %s\n",command,OZUPDATER_URL);    
#endif
    error = gnet_io_channel_writen(iochannel, command, strlen(command), &n);
    g_free(command);
    if(error != G_IO_ERROR_NONE) {
#ifdef DEBUG
	g_print("DEBUG: IO WRITE CHANNEL ERROR\n");    
#endif
        return;
    }    
    error = g_io_channel_read(iochannel, buffer, sizeof(buffer), &n);
    if(error != G_IO_ERROR_NONE) {
#ifdef DEBUG
	g_print("DEBUG: IO READ CHANNEL ERROR\n");    
#endif
        return;
    }     

#ifdef DEBUG
    g_print("DEBUG: Readed %d bytes\n",n);    
#endif
    
    XMLRoot = (struct _OzGtkXML *)OzGtkXML_ParseBuffer(g_locale_to_utf8(buffer,n,&bread,&bwrite,&utf8error),n);
    /* ORA INTERPRETA IL FILE */
    XMLTemp = (struct _OzGtkXML *)OzGtkXML_GetElementFromName(XMLRoot,OZUPDATER_PACKETNAME);
    if(XMLTemp != NULL) {
	rversion = g_strsplit((gchar *)OzGtkXML_GetElementValue(XMLTemp,"Version"),delim,5);
#ifdef DEBUG
	g_print("Latest Version: %s.%s.%s This version: %d.%d.%d\n",rversion[0],rversion[1],rversion[2],OZEDIT_VERSION,OZEDIT_SUBVERSION,OZEDIT_PATCH);
#endif
	if((atoi(rversion[0]) > OZEDIT_VERSION)||(atoi(rversion[1]) > OZEDIT_SUBVERSION)||(atoi(rversion[2]) > OZEDIT_PATCH)) {
	    topclass.need_update = TRUE;
	    g_sprintf(topclass.update_message,"OzEDITor v.%s has been released on %s.\nUpdate priority is %s.\nChanges:%s\n",(gchar *)OzGtkXML_GetElementValue(XMLTemp,"Version"),(gchar *)OzGtkXML_GetElementValue(XMLTemp,"ReleaseDate"),(gchar *)OzGtkXML_GetElementValue(XMLTemp,"Priority"),(gchar *)OzGtkXML_GetElementValue(XMLTemp,"Changes"));
	}
        g_strfreev(rversion);
    } else {
#ifdef DEBUG
	g_print("This package (%s) not supported by OzUPDATER !\n",OZUPDATER_PACKETNAME);    
#endif
    }
    gnet_tcp_socket_delete(socket);
#ifdef G_THREADS_ENABLED
    if(g_thread_supported()) {
	g_thread_exit(0);
    }
#endif    
#endif
}
